@@ -108,7 +108,7 @@ static PyObject *fparser_stop(PyObject *self, PyObject *args) {
 
     Py_BEGIN_ALLOW_THREADS
 
-    kill_parser(fparser->parser);
+    kill_parser(fparser->parser, 0);
 
     Py_END_ALLOW_THREADS
 
@@ -117,7 +117,11 @@ static PyObject *fparser_stop(PyObject *self, PyObject *args) {
 }
 
 static void fparser_dealloc(struct fparser *fparser) {
-    fparser_stop((PyObject *) fparser, NULL);
+    Py_BEGIN_ALLOW_THREADS
+    
+    kill_parser(fparser->parser, 1);
+    
+    Py_END_ALLOW_THREADS
 
     free(fparser->parser);
     kh_destroy(tcp_in_python, fparser->tcp_cache_table);
@@ -398,6 +402,7 @@ static PyObject *fparser_find_udp_flow(PyObject *self, PyObject *args) {
 
 static void fparser_flow_iter_dealloc(struct fparser_flow_iter *iter) {
     kv_destroy(iter->flows);
+    Py_DECREF(iter->fparser);
     iter->ob_type->tp_free((PyObject *) iter);
 }
 
@@ -442,6 +447,7 @@ static void fparser_tcp_flow_dealloc(struct fparser_tcp_flow *fparser_flow) {
         kh_del(tcp_in_python, table, k);
     }
 
+    Py_DECREF(fparser_flow->fparser);
     fparser_flow->ob_type->tp_free((PyObject *) fparser_flow);
 }
 
@@ -460,6 +466,7 @@ static void fparser_udp_flow_dealloc(struct fparser_udp_flow *fparser_flow) {
         kh_del(udp_in_python, table, k);
     }
 
+    Py_DECREF(fparser_flow->fparser);
     fparser_flow->ob_type->tp_free((PyObject *) fparser_flow);
 }
 
@@ -527,7 +534,9 @@ static PyObject *fparser_get_flow_iter(PyObject *self,
 
     kv_init(iter->flows);
     iter->curr_pos = 0;
+    
     iter->fparser = fparser;
+    Py_INCREF(iter->fparser);
 
     pthread_rwlock_rdlock(&p->flows_table_lock);
     DEBUG(p, "Acquired read lock on flows table");
@@ -1079,7 +1088,10 @@ static struct fparser_tcp_flow *tcp_flow_to_python(struct flow_id *id,
 
     py_flow->flow = flow;
     py_flow->id = id;
+    
     py_flow->fparser = fparser;
+    Py_INCREF(py_flow->fparser);
+    
     init_tcp_iter(py_flow);
 
     k = kh_put(tcp_in_python, fparser->tcp_cache_table, id, &ret);
@@ -1182,7 +1194,10 @@ static struct fparser_udp_flow *udp_flow_to_python(struct flow_id *id,
 
     py_flow->flow = flow;
     py_flow->id = id;
+    
     py_flow->fparser = fparser;
+    Py_INCREF(py_flow->fparser);
+    
     init_udp_iter(py_flow);
 
     k = kh_put(udp_in_python, fparser->udp_cache_table, id, &ret);
