@@ -53,13 +53,16 @@ void close_parser(struct parser_base *p) {
     }
 }
 
-void kill_parser(struct parser_base *p) {
+void kill_parser(struct parser_base *p, int dealloc_parser) {
     p->to_kill = 1;
     
     close_sniff(p);
     close_collector(p);
     close_dumper(p);
-    close_parser(p);
+    
+    if (dealloc_parser) {
+        close_parser(p);
+    }
     
     if (p->after_kill_callback) {
         p->after_kill_callback(p);
